from .tool import construct_range
from . import init_connection


def write(project, sheet_id, data):
    worksheet_name = data["worksheet_name"]
    columns_name = data["columns_name"]
    rows = data["rows"]
    values = [columns_name] + rows
    body = {
        "value_input_option": "USER_ENTERED",
        "data": [

            {
                "range": construct_range(worksheet_name, 'A1'),
                "majorDimension": "ROWS",
                "values": values,
            }
        ]
    }
    account = init_connection.get_api_account(project)
    response = account.values().batchUpdate(spreadsheetId=sheet_id,
                                            body=body).execute()
    return response



import googleapiclient

from . import write
from .manage_sheet import add_worksheet
from .read import check_availability_column


def send_to_sheet(project, sheet_id, data):
    try:
        available_response = check_availability_column(project, sheet_id, data)

    except googleapiclient.errors.HttpError:
        add_worksheet(project, sheet_id, data["worksheet_name"])
        available_response = {
            "completely_available": True
        }
    available = available_response["completely_available"]
    if not available:
        user_response = input("Columns " +
                              ", ".join(available_response["warning_columns"]) +
                              " will be overwritten, do you want to continue ?" +
                              "\nType yes to continue\n")
        if user_response.lower() not in ["yes", "y"]:
            return 0
    write.write(project, sheet_id, data)
    return 0



from . import init_connection


def get_sheet_info(project, sheet_id):
    account = init_connection.get_api_account(project)
    sheet = account.get(
        spreadsheetId=sheet_id
    ).execute()
    return sheet


def find_worksheet(project, sheet_id, worksheet_name):
    sheet = get_sheet_info(project, sheet_id)
    wks_list = sheet.get("sheets")
    for wks in wks_list:
        if wks.get("properties").get("title").lower() == worksheet_name.lower():
            return wks
    print("This worksheet doesn't exist")
    return None


def add_worksheet(project, sheet_id, worksheet_name):
    account = init_connection.get_api_account(project)
    requests = [
        {
            "addSheet": {
                "properties": {
                    "title": worksheet_name,
                    "tabColor": {
                        "red": 0.0,
                        "green": 0.0,
                        "blue": 1.0
                    }
                }
            }
        }
    ]
    body = {
        'requests': requests
    }
    response = account.batchUpdate(spreadsheetId=sheet_id,
                                 body=body).execute()
    return response



def construct_range(worksheet_name, start, end=None):
    range = "'" + worksheet_name + "'" + "!" + start
    if end:
        range = range + ":" + end
    return range


def column_string(n):
    string = ""
    while n > 0:
        n, remainder = divmod(n - 1, 26)
        string = chr(65 + remainder) + string
    return string


import pyred
from . import core


def query_to_sheet(project, sheet_id, worksheet_name, instance, query):
    items = pyred.execute.execute_query(instance, query)
    if not items:
        return 0
    columns_name = list(items[0].keys())
    rows = [list(i.values()) for i in items]
    data = {
        "worksheet_name": worksheet_name,
        "columns_name": columns_name,
        "rows": rows
    }
    core.send_to_sheet(project, sheet_id, data)
    return 0



import argparse
# Path to get client_secrets.json and to store credentials
import httplib2
from googleapiclient.discovery import build
from oauth2client import client, file
from oauth2client import tools

from pyspreadsheet.path import credential_path


def get_api_account(project, version='v4'):
    google_client_secret_path, google_credentials_path = credential_path(project)
    scopes = ['https://www.googleapis.com/auth/spreadsheets']
    discovery_uri = 'https://sheets.googleapis.com/$discovery/rest'
    parser = argparse.ArgumentParser(
        formatter_class=argparse.RawDescriptionHelpFormatter,
        parents=[tools.argparser])
    flags = parser.parse_args([])

    # Set up a Flow object to be used if we need to authenticate.
    flow = client.flow_from_clientsecrets(
        google_client_secret_path,
        scope=scopes,
        message=tools.message_if_missing(google_client_secret_path))

    path_storage = google_credentials_path + "/spreadsheet.json"
    storage = file.Storage(path_storage)
    credentials = storage.get()
    if credentials is None or credentials.invalid:
        credentials = tools.run_flow(flow, storage, flags)
    http = credentials.authorize(http=httplib2.Http())

    # Build the service object.
    if version == 'v3':
        account = build('analytics', version, http=http)
    else:
        account = build('analytics', version, http=http, discoveryServiceUrl=discovery_uri)

    return account.spreadsheets()



import os


def credential_path(project):
    google_client_secret_path = os.environ.get("GOOGLE_%s_CLIENT_SECRET_PATH" % project)
    google_credentials_path = os.environ.get("GOOGLE_%s_CREDENTIALS_PATH" % project)

    if not google_client_secret_path:
        google_client_secret_path = './'

    if not google_credentials_path:
        google_credentials_path = './'

    if google_client_secret_path[-1] == '/':
        google_client_secret_path = google_client_secret_path[:-1]

    if google_credentials_path[-1] == '/':
        google_credentials_path = google_credentials_path[:-1]

    google_client_secret_path = google_client_secret_path + '/client_secrets.json'
    return google_client_secret_path, google_credentials_path


import pandas

from .tool import column_string, construct_range
from .init_connection import get_api_account


def get_data_from_range(project, sheet_id, worksheet_name, nb_column, nb_row):
    range = construct_range(worksheet_name, "A1", column_string(nb_column) + str(nb_row + 1))
    account = get_api_account(project)
    data = account.values().get(
        spreadsheetId=sheet_id,
        range=range
    ).execute()
    return data


def check_first_row(values, columns_name):
    nb_column = len(columns_name)
    first_row = values[0]
    column_index_to_investigate = []
    for i in range(nb_column):
        try:
            column_name = first_row[i]
            if column_name != columns_name[i]:
                column_index_to_investigate.append(i)
        except IndexError:
            column_index_to_investigate.append(i)
    return column_index_to_investigate


def check_other_row(column_index_to_investigate, values):
    df = pandas.DataFrame(values[1:])
    warning_columns = []
    for i in column_index_to_investigate:
        try:
            boolean_list = list(pandas.isnull(df[i].replace(to_replace=[""], value=[None])))
            if not all(boolean_list):
                warning_columns.append(i)

        except KeyError:
            pass
    return warning_columns


def check_availability_column(project, sheet_id, data):
    worksheet_name = data["worksheet_name"]
    columns_name = data["columns_name"]
    rows = data["rows"]
    nb_row = len(rows)
    nb_column = len(columns_name)
    data = get_data_from_range(project, sheet_id, worksheet_name, nb_column, nb_row)
    completely_available = {
        "completely_available": True
    }
    values = data.get("values")
    if not values:
        return completely_available
    response = {
        "completely_available": False
    }
    column_index_to_investigate = check_first_row(values, columns_name)
    warning_columns = check_other_row(column_index_to_investigate, values)
    if not warning_columns:
        return completely_available
    warning_columns = [column_string(wc + 1) for wc in warning_columns]
    response["warning_columns"] = warning_columns
    return response
